/* $Id: rxhello7.cpp,v 1.4 1995/09/23 21:18:19 artc Exp $ */
/* 
      (C) Copyright 1993-1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

//
// rxhello7.cpp - Rx app to manipulate AutoCAD AcDb entities
//
// Jeremy Tammik, Autodesk ESC, 94-08-08
//
// The minimal Rx program manipulating AutoCAD entities
// using the Rx C++ API. All we do is create a line or
// circle entity and manipulate its start rep. center point.
//
// We define an AutoCAD command HELLO7 to do so.
//
// I have not yet figured out the UCS manipulation stuff
// perfectly, but it works fine in a plan view in WCS :-)
//
#include <string.h>
#include <aced.h>
#include <dbents.h>
#include <dbsymtb.h>
#include <adslib.h>
#include "rxhello7.h"

#define streq(s,t) (strcmp((s),(t))==0)

//
// hello7 - AutoCAD command to create new entity:
//
// We chose the simplest entity of them all ...
// the circle -- or should we create a line?
//
// All entites now belong to some block, since both
// model and paper space are also represented as blocks.
// To add an entity to the database, you first need to 
// get the block table from the database, then get the
// block from the block table, then get the current record
// from the block, and then add the entity to it.
//
// We create either a circle or a line entity, and then
// make use of polymorphism to handle them both as generic
// entites.
//
void
hello7()
{
    ads_printf("This is hello7() in RxHello7 ...\n");

    int rc;
    char kw[20];
    ads_initget(0, "Modify Circle Line");
    rc = ads_getkword("Modify group 10/Circle/<Line>: ", kw);

    if (rc == RTNONE) {
        strcpy(kw, "Line");
        rc = RTNORM;
    }
    if (rc != RTNORM) {
        ads_printf("Nothing selected.\n");
        return;
    }

    AcDbDatabase *pDb = acdbCurDwg();
    AcGeVector3d normal;
    getZucs(pDb, normal);

    AcGePoint3d p;
    AcDbEntity *pObj;
    if (streq(kw, "Line")) {

        // get line parameters from user:

        AcGePoint3d q;
        rc = getPoint("From point: ", p);
        if (rc != RTNORM) { return; }

        rc = getPoint(p, "To point: ", q);
        if (rc != RTNORM) { return; }

        // create a line:
        //
        // instantiate a new AcDbLine
        // set it to database defaults
        // set its normal vector
        // set its start point
        // set its endpoint

        AcDbLine *pLine = new AcDbLine;

        pLine->setDatabaseDefaults(pDb);

        // ... Next 3 lines fill in this ...
        pLine->setNormal(normal);
        pLine->setStartPoint(p);
        pLine->setEndPoint(q);

        pObj = pLine;

        // continue below to add obj to database ...
    } else if (streq(kw, "Circle")) {

        // get circle parameters from user:

        rc = getPoint("Centre point: ", p);
        if (rc != RTNORM) { return; }

        ads_point pucs;
        sa_w2u((double*)(&p), pucs);

        double r;
        rc = ads_getdist(pucs, "Radius: ", &r);
        if (rc != RTNORM) { return; }

        // create a circle:
        //
        // instantiate a new AcDbCircle
        // set it to database defaults
        // set its normal vector
        // set its centre point
        // set its radius

        AcDbCircle *pCircle = new AcDbCircle;

        pCircle->setDatabaseDefaults(pDb);

        // ...  next three lines fill in this ...
        pCircle->setNormal(normal);
        pCircle->setCenter(p);
        pCircle->setRadius(r);

        pObj = pCircle;

        // continue below to add obj to database ...
    } else if (streq(kw, "Modify")) {

        AcDbObjectId eId;
        pObj = selectEntity(eId, AcDb::kForWrite);
        if (!pObj) { return; }

        // if it is a circle or line,
        // modify start/center point:
        //
        AcDbLine *pLine;
        AcDbCircle *pCircle;
        if ((pCircle = AcDbCircle::cast(pObj)) != NULL) {
            // set p to the original circle centre:
            //
            // ... next line completes this ...
            p = pCircle->center();

            rc = getPoint(p, "New centre point: ", p);
            if (rc == RTNORM) { 
                // set the circle centre to the new point:
                //
                // ... next line completes this ...
                pCircle->setCenter(p);
            }
          pCircle->close();
        } else if ((pLine = AcDbLine::cast(pObj)) != NULL) {
            // set p to the original line start point:
            //
            // ... next line completes this ...
            p = pLine->startPoint();

            rc = getPoint(p, "New start point: ", p);
            if (rc == RTNORM) { 
                // set the line start point to the new point:
                //
                // ... next line completes this ...
                pLine->setStartPoint(p);
            }
            pLine->close();
        } else {
            ads_printf("Not a circle or line.\n");
        }
        return;
    }
    //
    // we are creating a new entity, either line or circle,
    // and now the generic AcDbEntity *pObj is pointing to it ...
    //
    // open block table to search for model space block:
    //

    AcDbBlockTable *pDbTable;
    Acad::ErrorStatus es = pDb->getBlockTable(pDbTable, AcDb::kForRead);
    if (es != Acad::eOk) {
        ads_printf("Unable to open block table.\n");
        if (pObj->close() != Acad::eOk) {
            acrx_abort("Unable to close entity.\n");
        }
        return;
    }
    
    // open current record in model space block:
    //
    AcDbBlockTableRecord *pDbRecord;
    es = pDbTable->getAt(ACDB_MODEL_SPACE, AcDb::kForWrite, pDbRecord);
    if (es != Acad::eOk) {
        ads_printf("Unable to open model space block.\n");
        if (pDbTable->close() != Acad::eOk) {
            acrx_abort("Unable to close block table.\n");
        }
        return;
    }
    
    // append new entity to model space block:
    //
    AcDbObjectId objId;
    es = pDbRecord->appendAcDbEntity(objId, pObj);
    if (es != Acad::eOk) {
        ads_printf("Unable to append entity.\n");
        if (pDbRecord->close() != Acad::eOk) {
            acrx_abort("Unable to close database record.\n");
        }
        return;
    }

    if (pObj->close() != Acad::eOk) {
        acrx_abort("Unable to close entity.\n");
    }

    if (pDbTable->close() != Acad::eOk) {
        acrx_abort("Unable to close block table.\n");
    }

    if (pDbRecord->close() != Acad::eOk) {
        acrx_abort("Unable to close database record.\n");
    }
}

// selectEntity - utility function to select entity
//
// Select entity, retrieve object Id and open it,
// creating and returning a valid object pointer.
//
AcDbEntity*
selectEntity(AcDbObjectId& eId, AcDb::OpenMode openMode)
{

    ads_name en;
    ads_point pt;
    int rc = ads_entsel("\nSelect an entity: ", en, pt);
    if (rc != RTNORM) {
        ads_printf("Nothing selected.\n", rc);
        return NULL;
    }
    
    Acad::ErrorStatus es = acdbGetObjectId(eId, en);
    if (es != Acad::eOk) {
        ads_printf("Either acdbGetObjectId or ads_entsel failed: "
            "Entity name <%lx,%lx>, error %d.\n", en[0], en[1], es);
        return NULL;
    }
    AcDbEntity* entObj;

    es = acdbOpenEntity(entObj, eId, openMode);
    if (es != Acad::eOk) {
        ads_printf("acdbOpenEntity failed with error %d.\n", es);
    return NULL;
    }
    return entObj;
}

// getZucs - retrieve UCS Z coordinate:
//
void
getZucs(AcDbDatabase * pDb, AcGeVector3d& v)
{
    AcGeVector3d x = pDb->pucsxdir();
    AcGeVector3d y = pDb->pucsydir();
    v = x.crossProduct(y);
    v.normalize();
}

// getPoint - acquire WCS point from user:
//
int
getPoint(AcGePoint3d& pF, char* pMessage, AcGePoint3d& pT)
{
    int rc;
    ads_point pFrom, pTo;

    if ((rc = sa_w2u((double*)(&pF), pFrom)) == RTNORM
        && (rc = ads_getpoint(pFrom, pMessage, pTo)) == RTNORM) 
    {
        rc = sa_u2w(pTo, (double*)(&pT));
    }
    return rc;
}

int
getPoint(char* pMessage, AcGePoint3d& pT)
{
    int rc;
    ads_point pTo;

    if ((rc = ads_getpoint(NULL, pMessage, pTo)) == RTNORM) {
        rc = sa_u2w(pTo, (double*)(&pT));
    }
    return rc;
}

int
sa_u2w(ads_point p1, ads_point p2)
{
    struct resbuf wcs,ucs;

    wcs.restype     = RTSHORT;
    wcs.resval.rint = 0;
    ucs.restype     = RTSHORT;
    ucs.resval.rint = 1;

    return ads_trans(p1, &ucs, &wcs, 0, p2);
}

int
sa_w2u(ads_point p1, ads_point p2)
{
    struct resbuf wcs,ucs;

    wcs.restype     = RTSHORT;
    wcs.resval.rint = 0;
    ucs.restype     = RTSHORT;
    ucs.resval.rint = 1;

    return ads_trans(p1, &wcs, &ucs, 0, p2);
}

void
initApp()
{
    ads_printf("Rx Hello 7 -- initApp.\n");
    acedRegCmds->addCommand("HELLO7", "RXHELLO7", "HELLO7",
        ACRX_CMD_MODAL, &hello7);
}

void
unloadApp()
{
    ads_printf("Rx Hello 7 -- unloadApp.\n");
    acedRegCmds->removeGroup("HELLO7");
}

extern "C" AcRx::AppRetCode
acrxEntryPoint(AcRx::AppMsgCode msg, void* pkt)
{
    switch (msg) {
    case AcRx::kInitAppMsg:
        acrxDynamicLinker->unlockApplication(pkt);
        initApp();
        break;
    case AcRx::kUnloadAppMsg:
        unloadApp();
        break;
    case AcRx::kLoadADSMsg:
        ads_printf("\nRx Hello 7 -- kLoadADSMsg\n");
        break;
    case AcRx::kUnloadADSMsg:
        ads_printf("Rx Hello 7 -- kUnloadADSMsg\n");
        break;
    default:
        ads_printf("Rx Hello 7 -- <unknown message %d>\n", msg);
        break;
    }
    return AcRx::kRetOK;
}

