/* $Id: ereact.cpp,v 1.2 1995/09/23 21:30:41 artc Exp $ */
/* 
      (C) Copyright 1993-1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

//
// ereact.cpp - Rx sample application for entity reactors
//
// Jeremy Tammik, Autodesk ESC, 94-08-23
//
// We create two line entites A and B and attach reactors
// to both of them. Whenever A's length changes, B is
// notified and changes its length. Whenever B's length
// changes, A is notified. Command options allow you to
// set up the reactors to
//  - make the lines the same length
//  - ask B to always make itself slightly longer than A
//  - switch on and off the ensuing (infitite) recursion
// The program should check for user interruption, by the
// way ... I guess using ads_usrbrk().
//
// Unfortunately, this enterprising attempt is doomed to
// fail: when I try to open object A inside the reactor
// which is called in reaction to the changes in B
// occasioned by the reactor called due to the original
// changes in A, acdbOpenEntity returns the error code
// eOpenForNotify ... which does make sense ...
//
// My custom reactor class does not have to be an Rx class,
// although it can be if I choose so. Yes?
//
// I seem to detect some strange behaviour when editing the
// two lines using grip points ... the modified line gets
// surprisingly long, but its grip points remain in the proper
// position, and still work ... ?
//
// I fixed the length problem by not using v.normalizeIt,
// and calling v.normal instead ... is there a bug in
// the former, or in Visual C++?
//
// I still get occasional very strange bahaviour of grip
// points, where the line has changed length, but its grip
// points remain where they were previously, and still work
// (more or less).
//
// For the Jreactor class, which notifies another entity when
// the entity it is attached to changes, a slightly simpler
// solution might be to use a persistent reactor instead.
// If you have finished the of implementing the non-persistent
// Jreactor code, go ahead and change it to use persistent
// reactors instead.
//
// Is it really ok to open an entity for read only in order to
// attach a persistent reactor? It seems to work ok ...
//
#include <string.h>
#include <stdlib.h>
#include <aced.h>
#include <dbents.h>
#include <dbsymtb.h>
#include <adslib.h>
#include "ereact.h"
#include "adeskabb.h"

#define streq(s,t) (strcmp((s),(t))==0)

void ereact()
{
    ads_printf("Welcome to the entity reactor sample application ...\n");

    char kw[20];
    ads_initget(0, "Messages Persistent Lines");
    int rc = ads_getkword("Messages/Persistent/<Lines>: ", kw);

    if (rc == RTNONE) {
        strcpy(kw, "Lines");
        rc = RTNORM;
    }
    if (rc != RTNORM) {
        ads_printf("Nothing selected.\n");
        return;
    }
    if (streq(kw, "Lines")) { createLines(); }
    else if (streq(kw, "Messages")) { attachMessages(); }
    else if (streq(kw, "Persistent")) { createPersistent(); }
}

// MsgReactor - customized entity reactor class which
//              prints out messages at all events:
//
class MsgReactor : public AcDbEntityReactor
{

public:
    MsgReactor() {}; // allow instantiation

    // AcDbObjectReactor protocol:
    //
    virtual void  cancelled(const AcDbObject* dbObj)
    {
        ads_printf("\nInside cancelled.\n");
    }

    virtual void  classChanged(const AcDbObject* dbObj,
        const AcDbObject* newObject)
    {
        ads_printf("\nInside classChanged.\n");
    }

    virtual void  copied(const AcDbObject* dbObj,
        const AcDbObject* newObj)
    {
        ads_printf("\nInside copied.\n");
    }

    virtual void  erased(const AcDbObject* dbObj,
        Adesk::Boolean pErasing = Adesk::kTrue)
    {
        ads_printf("\nInside erased.\n");
    }

    virtual void  goodbye(const AcDbObject* dbObj)
    {
        ads_printf("\nInside goodbye.\n");
    }

    virtual void  openedForModify(const AcDbObject* dbObj)
    {
        ads_printf("\nInside openedForModify.\n");
    }

    virtual void  openedForUndo(const AcDbObject* dbObj)
    {
        ads_printf("\nInside openedForUndo.\n");
    }

    virtual void  modified(const AcDbObject* dbObj)
    {
        ads_printf("\nInside modified.\n");
    }

    virtual void  modifyUndone(const AcDbObject* dbObj)
    {
        ads_printf("\nInside modifyUndone.\n");
    }

    virtual void  modifiedXData(const AcDbObject* dbObj,
        const AcRxSet* regappNames)
    {
        ads_printf("\nInside modifiedXData.\n");
    }

    virtual void  modifiedSubmodel(const AcDbObject* dbObj)
    {
        ads_printf("\nInside modifiedSubmodel.\n");
    }

    virtual void  transientUpdate(const AcDbObject* dbObj)
    {
        ads_printf("\nInside transientUpdate.\n");
    }

    // AcDbEntityReactor protocol:
    //
    virtual void  modifiedGraphics(const AcDbObject* dbObj)
        { ads_printf("\nInside modifiedGraphics.\n"); }

};


void attachMessages()
{
    AcDbEntity *pEnt;
    pEnt = selectEntity(AcDb::kForWrite);

    if (pEnt) {
        // instantiate a message reactor
        // attach it to an entity
        // don't forget to close!
        //
        // ... Next 3 lines complete this ...
        MsgReactor *pReactor = new MsgReactor();
        pEnt->addReactor(pReactor);
        pEnt->close();
    }
}

// Jreactor - customized entity reactor class which
// hooks up two lines:
//
class Jreactor : public AcDbEntityReactor
{
public:
    Jreactor(AcDbObjectId i, double f=1.0) { mId=i; mFactor=f; };
    virtual void modified(const AcDbObject* dbObj);

private:
    AcDbObjectId mId;
    double mFactor;
};

void Jreactor::modified(const AcDbObject* obj)
{
    AcDbLine *pLine = AcDbLine::cast(obj);
    if (!pLine) {
        const char *cstr = obj->isA()->name();
        ads_printf("This is a %s.\n", cstr);
        ads_printf("I only work with lines. Sorry.\n");
        return;
    }
    ads_printf("\nReactor attached to %lx calling %lx.\n",
    pLine->objectId(), mId);

    //
    // open object to update:
    //
    AcDbEntity *pEnt;
    Acad::ErrorStatus es = acdbOpenEntity(pEnt, mId, AcDb::kForWrite);
    if (es != Acad::eOk) {
        ads_printf("\nacdbOpenEntity failed with error %d%s.\n", es, es==83
            ? " (open for write)" : es==85 ? " (open for notify)" : "");
        return;
    }
    AcDbLine *pLine2 = AcDbLine::cast(pEnt);
    if (!pLine2) {
        const char *pCstr = pEnt->isA()->name();
        ads_printf("This reactor is attached to a %s.\n", pCstr);
        ads_printf("It should only be attached to lines. Sorry.\n");
        pEnt->close();
        return;
    }
    //
    // get length of modified entity:
    //
    AcGePoint3d p = pLine->startPoint();
    AcGePoint3d q = pLine->endPoint();
    AcGeVector3d v = q-p;
    double len = v.length();
    //
    // update other entity to match:
    //
    p = pLine2->startPoint();
    q = pLine2->endPoint();
    v = q-p;
    // v.normalizeIt(); // the debugger kept skipping this line!?
    // v *= len*factor;
    v = len * mFactor * v.normal();
    pLine2->setEndPoint(p+v);
    pLine2->close();
}

void createLines()
{
    AcDbDatabase *pDb = acdbCurDwg();

    AcDbLine *pLineA = new AcDbLine;
    pLineA->setDatabaseDefaults(pDb);
    pLineA->setStartPoint(AcGePoint3d(1,1,0));
    pLineA->setEndPoint(AcGePoint3d(2,1,0));

    AcDbObjectId aId;
    if (!append(pLineA, aId)) { return; }
    ads_printf("Line A is %lx from 1,1 to 2,1.\n",
    pLineA->objectId());

    AcDbLine *pLineB = new AcDbLine;
    pLineB->setDatabaseDefaults(pDb);
    pLineB->setStartPoint(AcGePoint3d(1,2,0));
    pLineB->setEndPoint(AcGePoint3d(2,2,0));

    AcDbObjectId bId;
    if (!append(pLineB, bId)) {
        pLineA->erase();
        pLineA->close();
        return;
    }
    ads_printf("Line B is %lx from 1,2 to 2,2.\n",
    pLineB->objectId());
    pLineB->close();

    // create a new Jreactor with B's object id
    // attach it to line A
    //
    // ... Next 3 lines complete this ...
    Jreactor *pRa = new Jreactor(bId);
    pLineA->addReactor(pRa);
    pLineA->close();

    // create a new Jreactor with A's object id
    // reopen line B
    // attach it to line B
    // don't forget to close things
    // ... Next 7 lines complete this...
    Jreactor *pRb = new Jreactor(aId, 1.1);
    if (acdbOpenEntity(pLineB, bId, AcDb::kForWrite) != Acad::eOk) {
        ads_printf("Error reopening second line.\n");
        return;
    }
    pLineB->addReactor(pRb);
    pLineB->close();
}

// Jpreactor - customized AcDbObject for persistent
//       reactor notification:
//
class Jpreactor : public AcDbObject
{
public:
    ACRX_DECLARE_MEMBERS(Jpreactor);
    Jpreactor() {}
    virtual void modified(const AcDbObject* dbObj);
};

ACRX_DXF_DEFINE_MEMBERS(Jpreactor, AcDbObject, 0, Jpreactor, Ereact);

void Jpreactor::modified(const AcDbObject * obj)
{
    const char * cstr = obj->isA()->name();
    ads_printf("\nThe %s with id %lx has been modified.\n",
        cstr, obj->objectId());
}

Adesk::Boolean
statOk(Acad::ErrorStatus stat)
{
    if (stat == Acad::eOk)
        return Adesk::kTrue;
    ads_printf("\nStatus Check Failed:  %d\n", stat);
    return Adesk::kFalse;
}

void
createPersistent()
{
    // create a customized object:
    //
    Jpreactor *pReactor = new Jpreactor();
    AcDbObjectId rId;

    // Open the namedObject Dictionary.  Check to see if there is an entry
    // under the name you wish to use for your storage dictionary.  If not,
    // then create a dictionary object and add it to the namedobject
    // Dictionary.  Then add your jpreactor object to this dictionary and set
    // rId to the objectId returned for it.
    //
    //...next 18 lines complete this...

    AcDbDictionary *pNameList;
    if (!statOk(acdbCurDwg()->getNamedObjectsDictionary(pNameList,
        AcDb::kForWrite)))
    {
        return;
    }

    AcDbObjectId myAppDictId;
    Acad::ErrorStatus es = pNameList->getAt("MkrTestDictionary", myAppDictId);
    if (es == Acad::eKeyNotFound) {
        AcDbDictionary *pDict = new AcDbDictionary;
        statOk(pNameList->setAt("MkrTestDictionary", pDict, myAppDictId));
        statOk(pDict->setAt("pers_react", pReactor, rId));
        statOk(pDict->close());
        statOk(pReactor->close());
    } 
    statOk(pNameList->close());

    AcDbEntity *pEnt = selectEntity(AcDb::kForWrite);

    if (pEnt) {
        // attach a persistent connection
        // from the Jpreactor object
        // to the selected entity:
        //
        //...next line completes...
        pEnt->addPersistentReactor(rId);


        es = pEnt->close();
        if (Acad::eOk != es) {
            acrx_abort("\nError %d closing entity.\n", es);
        }
    }
}

// append - append an opened entity to model space in the database:
//
Adesk::Boolean
append(AcDbEntity* pEntity)
{
    AcDbObjectId objId;
    return append(pEntity, objId);
}

Adesk::Boolean
append(AcDbEntity* pEntity, AcDbObjectId& objId)
{

    AcDbDatabase* pDb = acdbCurDwg();
    if (pDb == NULL) {
        ads_alert("No database!");
        return Adesk::kFalse;
    }
    
    AcDbBlockTable *pBlockTable;
    if (pDb->getBlockTable(pBlockTable, AcDb::kForRead) != Acad::eOk) {
        ads_alert("Failed to get block table!");
        return Adesk::kFalse;
    }
    
    AcDbBlockTableRecord *pBlockRec;
    if (pBlockTable->getAt(ACDB_MODEL_SPACE, AcDb::kForWrite, pBlockRec)
        != Acad::eOk)
    {
        ads_alert("Failed to get block table record!");
        pBlockTable->close();
        return Adesk::kFalse;
    }

    if (pBlockRec->appendAcDbEntity(objId, pEntity) != Acad::eOk) {
        ads_alert("Failed to append entity!");
        pBlockTable->close();
        pBlockRec->close();
        delete pEntity;
        return Adesk::kFalse;
    }
    pBlockRec->close();
    pBlockTable->close();
    return Adesk::kTrue;
}

// selectEntity - utility function to select entity
//
// Select entity, retrieve object Id and open it,
// creating and returning a valid object pointer.
//
AcDbEntity*
selectEntity(AcDb::OpenMode openMode)
{
    AcDbObjectId eId;
    return selectEntity(eId, openMode);
}

AcDbEntity*
selectEntity(AcDbObjectId& eId, AcDb::OpenMode openMode)
{
    ads_name en;
    ads_point pt;
    int rc = ads_entsel("\nSelect an entity: ", en, pt);

    if (rc != RTNORM) {
        ads_printf("Nothing selected.\n", rc);
        return NULL;
    }

    Acad::ErrorStatus es = acdbGetObjectId(eId, en);
    if (es != Acad::eOk) {
        ads_printf("Either acdbGetObjectId or ads_entsel failed: "
            "Entity name <%lx,%lx>, error %d.\n", en[0], en[1], es);
        return NULL;
    }
    AcDbEntity * ent;

    es = acdbOpenEntity(ent, eId, openMode);
    if (es != Acad::eOk) {
        ads_printf("acdbOpenEntity failed with error %d.\n", es);
        return NULL;
    }
    return ent;
}

// ----- Rx interface -----

void
initApp()
{
    ads_printf("Entity Reactor sample app -- initApp.\n");
    acedRegCmds->addCommand("EREACT", "EREACT", "EREACT",
        ACRX_CMD_MODAL, &ereact);
    Jpreactor::rxInit();
    acrxBuildClassHierarchy();
}

void
unloadApp()
{
    ads_printf("Entity Reactor sample app -- unloadApp.\n");
    acedRegCmds->removeGroup("EREACT");

    // Remove the Jpreactor class from the ACRX runtime class hierarchy.
    // If this is done while database is still active, it should cause
    // all objects of class Jpreactor to be zombified. 
    //
    deleteAcRxClass(Jpreactor::desc());
}

extern "C" AcRx::AppRetCode
acrxEntryPoint(AcRx::AppMsgCode msg, void* pkt)
{
    switch (msg) {
    case AcRx::kInitAppMsg:
        acrxDynamicLinker->unlockApplication(pkt);
        initApp();
        break;
    case AcRx::kUnloadAppMsg:
        unloadApp();
        break;
    case AcRx::kLoadADSMsg:
        ads_printf("\nEntity Reactor sample app -- kLoadADSMsg\n");
        break;
    case AcRx::kUnloadADSMsg:
        ads_printf("Entity Reactor sample app -- kUnloadADSMsg\n");
        break;
    default:
        ads_printf("Entity Reactor sample app -- <unknown message %d>\n", msg);
        break;
    }
    return AcRx::kRetOK;
}

