/* $Id: samp2.cpp,v 1.2 1995/09/23 21:37:34 artc Exp $ */
// samp2.cpp
// This program demonstrates what it takes to create a custom object class.
// Essentially,  this means deriving a class from AcDbObject and providing
// your own implementations of dwgInFields, dwgOutFields, dxfInFields, and
// dxfOutFields.

#include <rxobject.h>
#include <rxregsvc.h>
#include <aced.h>
#include <dbsymtb.h>
#include <adslib.h>


// function prototypes

void create_dictionary();
void iterate_dictionary();
void initApp();
void unloadApp();
extern "C" AcRx::AppRetCode acrxEntryPoint(AcRx::AppMsgCode, void*);

                       
// class declarations

class MyClass : public AcDbObject
//
// This is a custom object class to demonstrate what's necessary to
// make it work.
//
// To keep it simple,  this class has a single integer data member.
// Get and set functions are provided for this data member.
//
{
public:
    ACRX_DECLARE_MEMBERS(MyClass);
    MyClass() : mIntval(0){};
    MyClass(const int& val): mIntval(val) {};
    Acad::ErrorStatus           getData(int&);
    Acad::ErrorStatus           setData(const int&);
    virtual Acad::ErrorStatus   dwgInFields(AcDbDwgFiler* filer);
    virtual Acad::ErrorStatus   dwgOutFields(AcDbDwgFiler* filer) const;
    virtual Acad::ErrorStatus   dxfInFields(AcDbDxfFiler* filer);
    virtual Acad::ErrorStatus   dxfOutFields(AcDbDxfFiler* filer) const;
private:
    int mIntval;
};

ACRX_DXF_DEFINE_MEMBERS(MyClass, AcDbObject, 0, MYCLASS, SAMP2);


// gets value of integer data member
//
Acad::ErrorStatus
MyClass::getData(int& val)
{
    // let AutoCAD know that a read operation is taking place
    //
    assertReadEnabled();

    val = mIntval;
    return Acad::eOk;
}


// Sets value of integer data member
//
Acad::ErrorStatus
MyClass::setData(const int& val)
{
    // trigger openedForModify notification
    //
    assertWriteEnabled();

    mIntval = val;
    return Acad::eOk;
}


// Files data in from a DWG file
//
Acad::ErrorStatus
MyClass::dwgInFields(AcDbDwgFiler* filer)
{
    // among other things triggers openedForModify notification
    //
    assertWriteEnabled();

    AcDbObject::dwgInFields(filer);
    filer->readItem(&mIntval);
    return filer->filerStatus();
}


// Files data out to a DWG file
//
Acad::ErrorStatus
MyClass::dwgOutFields(AcDbDwgFiler* filer) const
{
    // let AutoCAD know that a read operation is taking place
    //
    assertReadEnabled();

    AcDbObject::dwgOutFields(filer);
    filer->writeItem(mIntval);
    return filer->filerStatus();
}


// Files data in from a DXF file
//
Acad::ErrorStatus
MyClass::dxfInFields(AcDbDxfFiler* filer)
{
    // among other things triggers openedForModify notification
    //
    assertWriteEnabled();

    Acad::ErrorStatus es;
    if ((es = AcDbObject::dxfInFields(filer)) != Acad::eOk) {
        return es;
    }

    // Check to see if we're at the right subclass getData marker
    //
    if (!filer->atSubclassData("MyClass")) {
        return Acad::eBadDxfSequence;
    }

    struct resbuf inbuf;

    while (es == Acad::eOk) {
        if ( (es = filer->readItem(&inbuf)) == Acad::eOk ) {
            if (inbuf.restype == AcDb::kDxfInt16) {
                    mIntval = inbuf.resval.rint;
            }
        }
    }
    return filer->filerStatus();
}


// Files data out to a DXF file
//
Acad::ErrorStatus
MyClass::dxfOutFields(AcDbDxfFiler* filer) const
{
    // let AutoCAD know that a read operation is taking place
    //
    assertReadEnabled();
    
    AcDbObject::dxfOutFields(filer);
    filer->writeItem(AcDb::kDxfSubclass, "MyClass");
    filer->writeItem(AcDb::kDxfInt16, mIntval);
    return filer->filerStatus();
}


// Creates two objects of class "myobject" fills them in with the integers
// 1 & 2 and then adds them to the dictionary associated with the key
// "TEST_DICT".  If this dictionary doesn't exist,  then it's created and
// added to the named objects dictionary.
//
void
create_dictionary()
{
    // Open the namedobjects dictionary for write
    //
    AcDbDictionary *pNamedobj;
    acdbCurDwg()->getNamedObjectsDictionary(pNamedobj, AcDb::kForWrite);

    // check to see if the dictionary we want to create is already present
    // if it's not,  then create it and add it to the namedobjects dictionary
    //
    AcDbDictionary *pDict;
    if (pNamedobj->getAt("TEST_DICT", AcDb::kForWrite,
        (AcDbObject*&) pDict) == Acad::eKeyNotFound)
    {
        pDict = new AcDbDictionary;
        AcDbObjectId DictId;
        pNamedobj->setAt("TEST_DICT", pDict, DictId);
    }
        
    // close the namedobjects dictionary
    //
    pNamedobj->close();

    if (pDict) {
        // create new objects to add to the new dictionary
        //
        MyClass *pObj1 = new MyClass(1),
                *pObj2 = new MyClass(2);

        AcDbObjectId rId1, rId2;

        // add the new objects to the dictionary
        //
        pDict->setAt("OBJ1", pObj1, rId1);
        pDict->setAt("OBJ2", pObj2, rId2);

        // close the new objects
        //
        pObj1->close();
        pObj2->close();

        // close the new dictionary
        //
        pDict->close();
    }
}


// Opens the dictionary associated with the key "TEST_DICT" and iterates
// through all it's entries printing out the integer data value in each
// entry.
//
void
iterate_dictionary()
{
    // Open the namedobjects dictionary for read
    //
    AcDbDictionary *pNamedobj;
    acdbCurDwg()->getNamedObjectsDictionary(pNamedobj, AcDb::kForRead);

    // get a pointer to the TEST_DICT dictionary
    //
    AcDbDictionary *pDict;
    pNamedobj->getAt("TEST_DICT", AcDb::kForRead, (AcDbObject*&) pDict);
        
    // finished with the namedobjects dictionary so close it
    //
    pNamedobj->close();

    // get an iterator for the TEST_DICT dictionary
    //
    AcDbDictionaryIterator *pDictIter = pDict->newIterator();

    MyClass *pMyCl;
    int val;
    for (;!pDictIter->done();pDictIter->nextElement()) {
          
        // get the current record and open it for read
        //
        pDictIter->getObject(AcDb::kForRead, (AcDbObject*&) pMyCl);

        // get the mIntval value
        //
        pMyCl->getData(val);

        // close the linetype record
        //
        pMyCl->close();
      
        // print the linetype name
        //
        ads_printf("\nintval is:  %d", val);
    }
 
    // delete the iterator
    //
    delete pDictIter;

    // close the dictionary
    //
    pDict->close();
}


// Initialization function called from acrxEntryPoint during kInitAppMsg
// case.  This function is used to add commands to the command stack and
// to add classes to the ACRX class hierarchy
//
void
initApp()
{
    acedRegCmds->addCommand("DICTIONARY_COMMANDS", "CREATE", "CREATE",
        ACRX_CMD_MODAL, &create_dictionary);

    acedRegCmds->addCommand("DICTIONARY_COMMANDS", "ITERATE", "ITERATE",
        ACRX_CMD_MODAL, &iterate_dictionary);

    MyClass::rxInit();
    acrxBuildClassHierarchy();
}


// Clean up function called from acrxEntryPoint during the kUnloadAppMsg
// case.  This function removes this apps command set from the command
// stack.
//
void
unloadApp()
{
    acedRegCmds->removeGroup("DICTIONARY_COMMANDS");
    
    // Remove the myObject class from the ACRX runtime class hierarchy.
    // If this is done while database is still active, it should cause
    // all objects of class myObject to be zombified. 
    //
    deleteAcRxClass(MyClass::desc());
}


// ARX entry point
//
AcRx::AppRetCode
acrxEntryPoint(AcRx::AppMsgCode msg, void* pkt)
{
    switch (msg) {
    case AcRx::kInitAppMsg:
        acrxDynamicLinker->unlockApplication(pkt);
        initApp();
        break;
    case AcRx::kUnloadAppMsg:
        unloadApp();
        break;
    case AcRx::kLoadADSMsg:
    case AcRx::kUnloadADSMsg:
    default:
        break;
    }
    return AcRx::kRetOK;
}

