#ifndef _AD_DBID_H
#define _AD_DBID_H 1
// 
// $Id: dbid.h,v 1.13 1995/10/12 13:39:55 t_lloyn Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc. 
// 
// The information contained herein is confidential, proprietary 
// to Autodesk,  Inc.,  and considered a trade secret as defined
// in section 499C of the penal code of the State of California. 
// Use of  this information  by  anyone  other  than  authorized
// employees of Autodesk, Inc.  is granted  only under a written 
// non-disclosure agreement,  expressly  prescribing  the  scope 
// and manner of such use. 
//
// CREATED BY:   Peter Janson, May, 1994
//
// DESCRIPTION:
//
// This file contains lightweight id classes distinguishing between
// different types of object id's. A distinction is made between an
// owned object and a reference or "pointer to" an object, as well as
// whether the relationship protects the object from purge (hard/soft).
//
// Hard-owner:  
//
// An owned object protected from purge. A layer symbol table and
// layer 0 is an example of a hard-owner relationship.
//
// Soft-owner:
//
// An owned object not protected from purge. All symbol tables own
// their records, but that relationship does not usually protect
// the records from purge.
//
// Hard-pointer:
//
// A reference to an object that is not owned, but protected from
// purge. An entity, for example, does not own the layer it is on
// but its reference to it protects it from purge.
//
// Soft-pointer:
//
// A reference to an object that is not owned, and not protected
// from purge. 

#include "adesk.h"

class AcDbStub;

class AcDbObjectId 
{
public:
    AcDbObjectId();
 
//  As long as the Visual C++ compiler refuses to inline the
//  following, remove it:
//  AcDbObjectId(const AcDbObjectId&);

    AcDbObjectId(const AcDbStub*);

    static const   AcDbObjectId  kNull;

    long           asOldId     () const;
    AcDbObjectId&  setFromOldId(long oldId);

    AcDbObjectId&  operator =  (const AcDbObjectId&);
    AcDbObjectId&  operator =  (const AcDbStub*);

    Adesk::Boolean operator <  (const AcDbObjectId&) const;
    Adesk::Boolean operator >  (const AcDbObjectId&) const;
    Adesk::Boolean operator >= (const AcDbObjectId&) const;
    Adesk::Boolean operator <= (const AcDbObjectId&) const;
    Adesk::Boolean operator == (const AcDbObjectId&) const;
    Adesk::Boolean operator != (const AcDbObjectId&) const;

    Adesk::Boolean operator <  (const AcDbStub*) const;
    Adesk::Boolean operator >  (const AcDbStub*) const;
    Adesk::Boolean operator >= (const AcDbStub*) const;
    Adesk::Boolean operator <= (const AcDbStub*) const;
    Adesk::Boolean operator == (const AcDbStub*) const;
    Adesk::Boolean operator != (const AcDbStub*) const;

#if defined(hpux) | defined(ibmrs) // cfront gets confused when comparing to constants without these:
    Adesk::Boolean operator == (int) const;
    Adesk::Boolean operator != (int) const;
#endif

    operator       AcDbStub*   () const;

protected:
    unsigned long  mId;
};


class AcDbHardOwnershipId : public AcDbObjectId
{
public:
    AcDbHardOwnershipId();
    AcDbHardOwnershipId(const AcDbObjectId&);
    AcDbHardOwnershipId(const AcDbStub*);

    AcDbHardOwnershipId& operator =(const AcDbHardOwnershipId&);
    AcDbHardOwnershipId& operator =(const AcDbObjectId&);
    AcDbHardOwnershipId& operator =(const AcDbStub*);

    operator AcDbStub* () const;
};


class AcDbSoftOwnershipId : public AcDbObjectId
{
public:
    AcDbSoftOwnershipId();
    AcDbSoftOwnershipId(const AcDbObjectId&);
    AcDbSoftOwnershipId(const AcDbStub*);

    AcDbSoftOwnershipId& operator =(const AcDbSoftOwnershipId&);
    AcDbSoftOwnershipId& operator =(const AcDbObjectId&);
    AcDbSoftOwnershipId& operator =(const AcDbStub*);

    operator AcDbStub* () const;
};


class AcDbHardPointerId : public AcDbObjectId
{
public:
    AcDbHardPointerId();
    AcDbHardPointerId(const AcDbObjectId&);
    AcDbHardPointerId(const AcDbStub*);

    AcDbHardPointerId& operator =(const AcDbHardPointerId&);
    AcDbHardPointerId& operator =(const AcDbObjectId&);
    AcDbHardPointerId& operator =(const AcDbStub*);

    operator AcDbStub* () const;
};


class AcDbSoftPointerId : public AcDbObjectId
{
public:
    AcDbSoftPointerId();
    AcDbSoftPointerId(const AcDbObjectId&);
    AcDbSoftPointerId(const AcDbStub*);
#ifndef sgi
    // There is no AcDbObjectId::operator= defined, ambiguous
    AcDbSoftPointerId& operator =(const AcDbSoftPointerId&);
#endif /* !sgi */
    AcDbSoftPointerId& operator =(const AcDbObjectId&);
    AcDbSoftPointerId& operator =(const AcDbStub*);

    operator AcDbStub* () const;
};


//////////////////// AcDbObjectId inlines ////////////////////

inline
AcDbObjectId::AcDbObjectId()
{ mId = 0; }

//inline
//AcDbObjectId::AcDbObjectId(const AcDbObjectId& id)
//{ mId = id.mId; }

inline
AcDbObjectId::AcDbObjectId(const AcDbStub* pStub)
{ mId = (unsigned long)(pStub); }

inline
long AcDbObjectId::asOldId() const
{ return mId; }

inline
AcDbObjectId& AcDbObjectId::setFromOldId(long oldId)
{ mId = oldId; return *this; }

inline 
AcDbObjectId& AcDbObjectId::operator = (const AcDbObjectId& id)
{ mId = id.mId; return *this; }

inline 
AcDbObjectId& AcDbObjectId::operator = (const AcDbStub* pStub)
{ mId = (unsigned long)(pStub); return *this; }

inline
AcDbObjectId::operator AcDbStub* () const
{ return (AcDbStub*)(mId); }

//////////////////// AcDbHardOwnershipId inlines ////////////////////

inline
AcDbHardOwnershipId::AcDbHardOwnershipId()
{}

inline
AcDbHardOwnershipId::AcDbHardOwnershipId(const AcDbObjectId& id)
  : AcDbObjectId(id)
{}

inline
AcDbHardOwnershipId::AcDbHardOwnershipId(const AcDbStub* pStub)
  : AcDbObjectId(pStub)
{}

inline
AcDbHardOwnershipId& AcDbHardOwnershipId::operator 
= (const AcDbHardOwnershipId& id)
{ AcDbObjectId::operator=(id); return *this; }

inline
AcDbHardOwnershipId& AcDbHardOwnershipId::operator = (const AcDbObjectId& id)
{ AcDbObjectId::operator=(id); return *this; }

inline 
AcDbHardOwnershipId& AcDbHardOwnershipId::operator = (const AcDbStub* pStub)
{ mId = (unsigned long)pStub; return *this; }

inline
AcDbHardOwnershipId::operator AcDbStub* () const
{ return (AcDbStub*)(mId); }


//////////////////// AcDbSoftOwnershipId inlines ////////////////////

inline
AcDbSoftOwnershipId::AcDbSoftOwnershipId()
{}

inline
AcDbSoftOwnershipId::AcDbSoftOwnershipId(const AcDbObjectId& id)
  : AcDbObjectId(id)
{}

inline
AcDbSoftOwnershipId::AcDbSoftOwnershipId(const AcDbStub* pStub)
  : AcDbObjectId(pStub)
{}

inline
AcDbSoftOwnershipId& AcDbSoftOwnershipId::operator 
= (const AcDbSoftOwnershipId& id)
{ AcDbObjectId::operator=(id); return *this; }

inline
AcDbSoftOwnershipId& AcDbSoftOwnershipId::operator = (const AcDbObjectId& id)
{ AcDbObjectId::operator=(id); return *this; }

inline 
AcDbSoftOwnershipId& AcDbSoftOwnershipId::operator = (const AcDbStub* pStub)
{ mId = (unsigned long)(pStub); return *this; }

inline
AcDbSoftOwnershipId::operator AcDbStub* () const
{ return (AcDbStub*)(mId); }


//////////////////// AcDbHardPointerId inlines ////////////////////

inline
AcDbHardPointerId::AcDbHardPointerId()
{}

inline
AcDbHardPointerId::AcDbHardPointerId(const AcDbObjectId& id)
  : AcDbObjectId(id)
{}

inline
AcDbHardPointerId::AcDbHardPointerId(const AcDbStub* pStub)
  : AcDbObjectId(pStub)
{}

inline
AcDbHardPointerId& AcDbHardPointerId::operator 
= (const AcDbHardPointerId& id)
{ AcDbObjectId::operator=(id); return *this; }

inline
AcDbHardPointerId& AcDbHardPointerId::operator = (const AcDbObjectId& id)
{ AcDbObjectId::operator=(id); return *this; }

inline 
AcDbHardPointerId& AcDbHardPointerId::operator = (const AcDbStub* pStub)
{ mId = (unsigned long)pStub; return *this; }

inline
AcDbHardPointerId::operator AcDbStub* () const
{ return (AcDbStub*)(mId); }


//////////////////// AcDbSoftPointerId inlines ////////////////////

inline
AcDbSoftPointerId::AcDbSoftPointerId()
{}

inline
AcDbSoftPointerId::AcDbSoftPointerId(const AcDbObjectId& id)
  : AcDbObjectId(id)
{}

inline
AcDbSoftPointerId::AcDbSoftPointerId(const AcDbStub* pStub)
  : AcDbObjectId(pStub)
{}

#ifndef sgi
// There is no AcDbObjectId::operator= defined, ambiguous
inline
AcDbSoftPointerId& AcDbSoftPointerId::operator = (const AcDbSoftPointerId& id)
{ AcDbObjectId::operator=(id); return *this; }
#endif /* !sgi */

inline
AcDbSoftPointerId& AcDbSoftPointerId::operator = (const AcDbObjectId& id)
{ AcDbObjectId::operator=(id); return *this; }

inline 
AcDbSoftPointerId& AcDbSoftPointerId::operator = (const AcDbStub* pStub)
{ mId = (unsigned long)pStub; return *this; }

inline
AcDbSoftPointerId::operator AcDbStub* () const
{ return (AcDbStub*)(mId); }


#endif // _AD_DBID_H
