#ifndef AC_GEMAT3D_H
#define AC_GEMAT3D_H
//
// $Id: gemat3d.h,v 1.4 1995/12/20 15:17:27 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement, expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class AcGeMatrix3d - a matrix used to
// represent general purpose 3d transformations in homogeneous
// co-ordinate space.

#include "gegbl.h"
#include "gemat2d.h"
#include "gepnt3d.h"

class AcGeLine3d;
class AcGeVector3d;
class AcGePlane;
class AcGeTol;

class AcGeMatrix3d
{
public:
    AcGeMatrix3d();
    AcGeMatrix3d(const AcGeMatrix3d&);

    // The multiplicative identity.
    //
    static const   AcGeMatrix3d    kIdentity;

    // Reset matrix.
    //
    AcGeMatrix3d&  setToIdentity();

    // Multiplication.
    //
    AcGeMatrix3d   operator *      (const AcGeMatrix3d&) const;
    AcGeMatrix3d&  operator *=     (const AcGeMatrix3d&);
    AcGeMatrix3d&  preMultBy       (const AcGeMatrix3d& leftSide);
    AcGeMatrix3d&  postMultBy      (const AcGeMatrix3d& rightSide);
    AcGeMatrix3d&  setToProduct    (const AcGeMatrix3d&, const AcGeMatrix3d&);

    // Multiplicative inverse.
    //
    AcGeMatrix3d&  invert          ();
    AcGeMatrix3d   inverse         () const;

    // Test if it is a singular matrix.  A singular matrix is not invertable.
    // 
    Adesk::Boolean isSingular      (const AcGeTol& = AcGeContext::gTol) const;

    // Matrix transposition.
    //
    AcGeMatrix3d&  transposeIt     ();
    AcGeMatrix3d   transpose       () const;

    // Tests for equivalence using the infinity norm.
    //
    Adesk::Boolean operator ==     (const AcGeMatrix3d&) const;
    Adesk::Boolean operator !=     (const AcGeMatrix3d&) const;
    Adesk::Boolean isEqualTo       (const AcGeMatrix3d&, const AcGeTol&
				    = AcGeContext::gTol) const;

    // Test scaling effects of matrix
    //
    Adesk::Boolean isUniScaledOrtho(const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isScaledOrtho   (const AcGeTol& = AcGeContext::gTol) const;

    // Determinant
    //
    double         det             () const;

    // Set/retrieve translation.
    //
    AcGeMatrix3d&  setTranslation  (const AcGeVector3d&);
    AcGeVector3d   translation     () const;

    // Set/get coordinate system
    //
    AcGeMatrix3d&  setCoordSystem  (const AcGePoint3d& origin,
                                    const AcGeVector3d& xAxis,
                                    const AcGeVector3d& yAxis,
                                    const AcGeVector3d& zAxis);
    void           getCoordSystem  (AcGePoint3d& origin, 
                                    AcGeVector3d& xAxis, 
                                    AcGeVector3d& yAxis, 
                                    AcGeVector3d& zAxis) const;

    // Set the matrix to be a specified transformation 
    //
    AcGeMatrix3d&  setToTranslation(const AcGeVector3d&);
    AcGeMatrix3d&  setToRotation   (double angle, const AcGeVector3d& axis,
                                    const AcGePoint3d& center
				    = AcGePoint3d::kOrigin);
    AcGeMatrix3d&  setToScaling    (double scaleAll, const AcGePoint3d& center
				    = AcGePoint3d::kOrigin);
    AcGeMatrix3d&  setToMirroring  (const AcGePlane&);
    AcGeMatrix3d&  setToMirroring  (const AcGePoint3d&);
    AcGeMatrix3d&  setToMirroring  (const AcGeLine3d&);
    AcGeMatrix3d&  setToProjection (const AcGePlane& projectionPlane,
                                    const AcGeVector3d& projectDir);
    AcGeMatrix3d&  setToAlignCoordSys(const AcGePoint3d& fromOrigin,
                                    const AcGeVector3d& fromXAxis,
                                    const AcGeVector3d& fromYAxis,
                                    const AcGeVector3d& fromZAxis,
                                    const AcGePoint3d& toOrigin,
                                    const AcGeVector3d& toXAxis,
                                    const AcGeVector3d& toYAxis,
                                    const AcGeVector3d& toZAxis);

    AcGeMatrix3d&  setToWorldToPlane(const AcGeVector3d& normal);
    AcGeMatrix3d&  setToWorldToPlane(const AcGePlane&);
    AcGeMatrix3d&  setToPlaneToWorld(const AcGeVector3d& normal);
    AcGeMatrix3d&  setToPlaneToWorld(const AcGePlane&);

    // Similar to above, but creates matrix on the stack.
    //
    static
    AcGeMatrix3d   translation     (const AcGeVector3d&);
    static
    AcGeMatrix3d   rotation        (double angle, const AcGeVector3d& axis,
				    const AcGePoint3d& center
				    = AcGePoint3d::kOrigin);
    static
    AcGeMatrix3d   scaling         (double scaleAll, const AcGePoint3d& center
				    = AcGePoint3d::kOrigin);
    static
    AcGeMatrix3d   mirroring       (const AcGePlane&);
    static
    AcGeMatrix3d   mirroring       (const AcGePoint3d&);
    static
    AcGeMatrix3d   mirroring       (const AcGeLine3d&);
    static
    AcGeMatrix3d   projection      (const AcGePlane& projectionPlane,
				    const AcGeVector3d& projectDir);
    static
    AcGeMatrix3d   alignCoordSys   (const AcGePoint3d&  fromOrigin,
				    const AcGeVector3d& fromXAxis,
				    const AcGeVector3d& fromYAxis,
				    const AcGeVector3d& fromZAxis,
				    const AcGePoint3d&  toOrigin,
				    const AcGeVector3d& toXAxis,
				    const AcGeVector3d& toYAxis,
				    const AcGeVector3d& toZAxis);

    static
    AcGeMatrix3d   worldToPlane    (const AcGeVector3d& normal);
    static
    AcGeMatrix3d   worldToPlane    (const AcGePlane&);
    static
    AcGeMatrix3d   planeToWorld    (const AcGeVector3d& normal);
    static
    AcGeMatrix3d   planeToWorld    (const AcGePlane&);

    // For convenient access to the data.
    //
    double         operator ()     (unsigned int, unsigned int) const;
    double&        operator ()     (unsigned int, unsigned int);

    // The components of the matrix.
    //
    double         entry[4][4];    // [row][column]

private:
    void           pivot           (int, AcGeMatrix3d&);
    int            pivotIndex(int) const;
    void           swapRows        (int, int, AcGeMatrix3d&);
};

// Return the element in position [row][column] of the `entry' array.
//
inline double
AcGeMatrix3d::operator () (
    unsigned int row,
    unsigned int column) const
{
    return entry[row][column];
}

inline double&
AcGeMatrix3d::operator () (
    unsigned int row,
    unsigned int column)
{
    return entry[row][column];
}

#endif
