#ifndef AC_GEPNT3D_H
#define AC_GEPNT3D_H
//
// $Id: gepnt3d.h,v 1.3 1995/12/20 15:17:47 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class AcGePoint3d - A mathematical entity
// used to represent a location in 3-space using homogeneous
// co-ordinates (implicitly).

#include "gevec3d.h"

class AcGeMatrix3d;
class AcGeLinearEnt3d;
class AcGePlane;
class AcGePlanarEnt;
class AcGeVector3d;
class AcGePoint2d;

class AcGePoint3d
{
public:
    AcGePoint3d();
    AcGePoint3d(const AcGePoint3d&);
    AcGePoint3d(double, double, double);
    AcGePoint3d(const AcGePlanarEnt&, const AcGePoint2d&); 

    // The origin, or (0, 0, 0).
    //
    static const   AcGePoint3d    kOrigin;

    // Matrix multiplication.
    //
    friend
    AcGePoint3d    operator *  (const AcGeMatrix3d&, const AcGePoint3d&);
    AcGePoint3d&   setToProduct(const AcGeMatrix3d&, const AcGePoint3d&);

    AcGePoint3d&   transformBy (const AcGeMatrix3d& leftSide);
    AcGePoint3d&   rotateBy    (double ang , const AcGeVector3d&,
                                const AcGePoint3d& = AcGePoint3d::kOrigin);       
    AcGePoint3d&   mirror      (const AcGePlane&);
    AcGePoint3d&   scaleBy     (double scaleFactor, const AcGePoint3d& 
                                wrtPoint = AcGePoint3d::kOrigin);
    AcGePoint2d    convert2d   (const AcGePlanarEnt&) const;

    // Scale multiplication.
    //
    AcGePoint3d    operator *  (double) const;
    friend
    AcGePoint3d    operator *  (double, const AcGePoint3d&);
    AcGePoint3d&   operator *= (double);
    AcGePoint3d    operator /  (double) const;
    AcGePoint3d&   operator /= (double);

    // Translation by a vector.
    //
    AcGePoint3d    operator +  (const AcGeVector3d&) const;
    AcGePoint3d&   operator += (const AcGeVector3d&);
    AcGePoint3d    operator -  (const AcGeVector3d&) const;
    AcGePoint3d&   operator -= (const AcGeVector3d&);
    AcGePoint3d&   setToSum    (const AcGePoint3d&, const AcGeVector3d&);

    // Get the vector between two points.
    //
    AcGeVector3d   operator -  (const AcGePoint3d&) const;
    AcGeVector3d   asVector    () const;

    // Distance to other geometric objects.
    //
    double         distanceTo       (const AcGePoint3d&) const;

    // Projection on plane
    //
    AcGePoint3d    project       (const AcGePlane&, const AcGeVector3d&) const;
    AcGePoint3d    orthoProject  (const AcGePlane&) const;
    
    // Tests for equivalence using the Euclidean norm.
    //
    Adesk::Boolean operator == (const AcGePoint3d&) const;
    Adesk::Boolean operator != (const AcGePoint3d&) const;
    Adesk::Boolean isEqualTo   (const AcGePoint3d&,
				const AcGeTol& = AcGeContext::gTol) const;

    // For convenient access to the data.
    //
    double         operator [] (unsigned int) const;
    double&        operator [] (unsigned int);
    AcGePoint3d&   set         (double, double, double);
    AcGePoint3d&   set         (const AcGePlanarEnt&, const AcGePoint2d&); 

    // The co-ordinates of the point.
    //
    double         x, y, z;
};

// Creates a point at the origin.
//
inline
AcGePoint3d::AcGePoint3d() : x(0.0), y(0.0), z(0.0)
{
}

// Creates a point with the same values as `src'.
//
inline
AcGePoint3d::AcGePoint3d(const AcGePoint3d& src) : x(src.x), y(src.y), z(src.z)
{
}

// Creates a point intialized to ( xx, yy, zz ).
//
inline
AcGePoint3d::AcGePoint3d(double xx, double yy, double zz) : x(xx), y(yy), z(zz)
{
}

// Returns a point such that each of the coordinates of this point
// have been multiplied by val.
//
inline AcGePoint3d
AcGePoint3d::operator * (double val) const
{
    return AcGePoint3d(x*val, y*val, z*val);
}

// Returns a point such that each of the coordinates of this point
// have been multiplied by val.
//
inline AcGePoint3d
operator * (double val, const AcGePoint3d& p)
{
    return AcGePoint3d(p.x*val, p.y*val, p.z*val);
}

// This is equivalent to the statement `p = p * val;'
// Each coordinate of this point is multiplied by val.
//
inline AcGePoint3d&
AcGePoint3d::operator *= (double val)
{
    x *= val;
    y *= val;
    z *= val;
    return *this;
}

// Returns a point such that each of the coordinates of this point
// have been divided by val.
//
inline AcGePoint3d
AcGePoint3d::operator / (double val) const
{
    return AcGePoint3d (x/val, y/val, z/val);
}

// This is equivalent to the statement `p = p / val;'
// Each coordinate of this point is divided by val.
//
inline AcGePoint3d&
AcGePoint3d::operator /= (double val)
{
    x /= val;
    y /= val;
    z /= val;
    return *this;
}

// Returns a point that is equivalent to the result of translating
// this point by the vector `v'.  (It yields the same result as if
// the vector had been cast to a translation matrix and then
// multiplied with the point.)
//
inline AcGePoint3d
AcGePoint3d::operator + (const AcGeVector3d& v) const
{
    return AcGePoint3d (x + v.x, y + v.y, z + v.z);
}

// This is equivalent to the statement `p = p + v;'
//
inline AcGePoint3d&
AcGePoint3d::operator += (const AcGeVector3d& v)
{
    x += v.x;
    y += v.y;
    z += v.z;
    return *this;
}

// This is equivalent to the statement `p + (-v);'
//
inline AcGePoint3d
AcGePoint3d::operator - (const AcGeVector3d& v) const
{
    return AcGePoint3d (x - v.x, y - v.y, z - v.z);
}

// This is equivalent to the statement `p = p - v;'
//
inline AcGePoint3d&
AcGePoint3d::operator -= (const AcGeVector3d& v)
{
    x -= v.x;
    y -= v.y;
    z -= v.z;
    return *this;
}

// This operator returns a vector such that if `v = p1 - p0',
// then, `v' is equivalent to the translation that takes
// `p0' to `p1'.  (This point is `p1').
//
inline AcGeVector3d
AcGePoint3d::operator - (const AcGePoint3d& p) const
{
    return AcGeVector3d (x - p.x, y - p.y, z - p.z);
}

// This operator returns the vector that would have resulted
// from the operation `p1 - AcGePoint3d::kOrigin', which is
// a common operation to perform.
//
inline AcGeVector3d
AcGePoint3d::asVector() const
{
    return AcGeVector3d(x, y, z);
}

// Sets the point to ( xx, yy, zz ).
//
inline AcGePoint3d&
AcGePoint3d::set(double xx, double yy, double zz)
{
    x = xx;
    y = yy;
    z = zz;
    return *this;
}

// Indexes the point as if it were an array.  `x' is index `0',
// `y' is index `1', `z' is index `2'.
//
inline double
AcGePoint3d::operator [] (unsigned int i) const
{
    return *(&x+i);
}

inline double&
AcGePoint3d::operator [] (unsigned int i)
{
    return *(&x+i);
}

#endif
