#ifndef AC_GESPHERE_H
#define AC_GESPHERE_H
//
// $Id: gesphere.h,v 1.7 1995/10/26 16:36:31 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// Description:
//
// This file implements the class AcGeImpSphere, a representation for a
// sphere.  It is defined by its radius, and the center. Two vectors,
// northAxis and refAxis are used in the parameterization of
// the sphere.  northAxis identifies the north and the south poles
// of the sphere. refAxis serves as a reference for the parameters.
// It is a vector orthogonal to northAxis.
// Parameter u is the longitude, which for a closed sphere defaults
// to the range [-PI, PI).  Zero correspond to the meridian that
// contains the refAxis of the sphere.
// Parameter v is the latitude, which for a closed sphere defaults
// to the range [-PI/2, PI/2].  The lower bound maps to the south
// pole, zero is on the equator, and the upperbound maps to the north
// pole.
// The sphere is periodic in u with a period of 2PI.
// [umin, umax] x [vmin, vmax] defines a four sided spherical patch
// bounded by two arcs that are meridians of longitude, and two arcs
// that are parallels of latitude.  Following constraints apply when
// defining a spherical patch.
// umin < umax and |umin - umax| <= 2PI.
// vmin < vmax and |vmin - vmax| <= PI. 
// 


#include "gegbl.h"
#include "gepnt3d.h"
#include "gevec3d.h"
#include "geintrvl.h"
#include "gesurf.h"

class AcGeCircArc3d;

class AcGeSphere : public AcGeSurface
{
public:
    AcGeSphere();
    AcGeSphere(double, const AcGePoint3d&);
    AcGeSphere(double, const AcGePoint3d&,
               const AcGeVector3d& northAxis, const AcGeVector3d& refAxis,
               double startAngleU, double endAngleU,
               double startAngleV, double endAngleV); 
    AcGeSphere(const AcGeSphere&);

    // Geometric properties.
    //
    double         radius            () const;
    AcGePoint3d    center            () const;
    void           getAnglesInU      (double& start, double& end) const;
    void           getAnglesInV      (double& start, double& end) const;
    AcGeVector3d   northAxis         () const;
    AcGeVector3d   refAxis           () const;
    AcGePoint3d    northPole         () const;
    AcGePoint3d    southPole         () const;
    Adesk::Boolean isOuterNormal     () const;
	Adesk::Boolean isClosedInU       (const AcGeTol& 
									   = AcGeContext::gTol) const;
  
    AcGeSphere&    setRadius         (double);
    AcGeSphere&    setAnglesInU      (double start, double end);
    AcGeSphere&    setAnglesInV      (double start, double end);
    AcGeSphere&    set               (double, const AcGePoint3d&);
    AcGeSphere&    set               (double, const AcGePoint3d&,
                                      const AcGeVector3d& northAxis,
                                      const AcGeVector3d& refAxis,
                                      double startAngleU,
                                      double endAngleU,
                                      double startAngleV,
                                      double endAngleV);
    // Assignment operator.
    //
    AcGeSphere&    operator =     (const AcGeSphere&);

    // Intersection with a linear entity
    //
    Adesk::Boolean intersectWith  (const AcGeLinearEnt3d&, int& intn, 
                                   AcGePoint3d& p1, AcGePoint3d& p2, 
                                   const AcGeTol& = AcGeContext::gTol) const;


protected:
    friend class AcGeImpSphere;

    // Construct object from its corresponding implementation object.
    AcGeSphere (AcGeImpSphere&, int);
    AcGeSphere (AcGeImpSphere*);
};

#endif
