#ifndef AC_GEVEC3D_H
#define AC_GEVEC3D_H
//
// $Id: gevec3d.h,v 1.4 1995/12/20 15:18:00 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class AcGeVector3d - a mathematical entity
// used to represent a direction and magnitude in 3-space, or
// alternatively to represent a linear translation in 3-space.

#include "adesk.h"
#include "gegbl.h"

class AcGeMatrix3d;
class AcGeVector2d;
class AcGePlane;
class AcGePlanarEnt;

class AcGeVector3d
{
public:
    AcGeVector3d();
    AcGeVector3d(const AcGeVector3d&);
    AcGeVector3d(double, double, double);
    AcGeVector3d(const AcGePlanarEnt&, const AcGeVector2d&); 

    // The additive identity, x-axis, y-axis, and z-axis.
    //
    static const   AcGeVector3d kIdentity;
    static const   AcGeVector3d kXAxis;
    static const   AcGeVector3d kYAxis;
    static const   AcGeVector3d kZAxis;

    // Multiplication with matrices.
    //
    friend
    AcGeVector3d   operator *  (const AcGeMatrix3d&, const AcGeVector3d&);
    AcGeVector3d&  setToProduct(const AcGeMatrix3d&, const AcGeVector3d&);

    AcGeVector3d&  transformBy (const AcGeMatrix3d& leftSide);
    AcGeVector3d&  rotateBy    (double ang , const AcGeVector3d& axis );
    AcGeVector3d&  mirror      (const AcGeVector3d& normalToPlane);
    AcGeVector2d   convert2d   (const AcGePlanarEnt&) const;

    // Multiplication by scalar.
    //
    AcGeVector3d   operator *  (double) const;
    friend
    AcGeVector3d   operator *  (double, const AcGeVector3d&);
    AcGeVector3d&  operator *= (double);
    AcGeVector3d&  setToProduct(const AcGeVector3d&, double);
    AcGeVector3d   operator /  (double) const;
    AcGeVector3d&  operator /= (double);

    // Addition and subtraction of vectors.
    //
    AcGeVector3d   operator +  (const AcGeVector3d&) const;
    AcGeVector3d&  operator += (const AcGeVector3d&);
    AcGeVector3d   operator -  (const AcGeVector3d&) const;
    AcGeVector3d&  operator -= (const AcGeVector3d&);
    AcGeVector3d&  setToSum    (const AcGeVector3d&, const AcGeVector3d&);
    AcGeVector3d   operator -  () const;
    AcGeVector3d&  negate      ();

    // Perpendicular vector
    //
    AcGeVector3d   perpVector  () const;

    // Angle argument.
    //
    double         angleTo     (const AcGeVector3d&) const;
    double         angleTo     (const AcGeVector3d&,
                                const AcGeVector3d& refVec) const;
    double         angleOnPlane(const AcGePlanarEnt&) const;

    // Vector length operations.
    //
    AcGeVector3d   normal      () const;
    AcGeVector3d&  normalize   ();
    double         length      () const;
    double         lengthSqrd  () const;
    Adesk::Boolean isUnitLength(const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isZeroLength(const AcGeTol& = AcGeContext::gTol) const;

    // Direction tests.
    //
    Adesk::Boolean isParallelTo(const AcGeVector3d&,
				const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isCodirectionalTo(const AcGeVector3d&,
				     const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isPerpendicularTo(const AcGeVector3d&,
				     const AcGeTol& = AcGeContext::gTol) const;

    // Dot and Cross product.
    //
    double         dotProduct  (const AcGeVector3d&) const;
    AcGeVector3d   crossProduct(const AcGeVector3d&) const;

    // Calculate rotation.
    //
    AcGeMatrix3d   rotateTo    (const AcGeVector3d&, const AcGeVector3d& axis
                                = AcGeVector3d::kIdentity) const;

    // Projection on plane
    //
    AcGeVector3d   project      (const AcGeVector3d& planeNormal, 
                                 const AcGeVector3d& projectDirection) const;
    AcGeVector3d   orthoProject (const AcGeVector3d& planeNormal) const;

    // Tests for equivalence using the Euclidean norm.
    //
    Adesk::Boolean operator == (const AcGeVector3d&) const;
    Adesk::Boolean operator != (const AcGeVector3d&) const;
    Adesk::Boolean isEqualTo   (const AcGeVector3d&,
				const AcGeTol& = AcGeContext::gTol) const;

    // For convenient access to the data.
    //
    double         operator [] (unsigned int) const;
    double&        operator [] (unsigned int);
    unsigned int   largestElement() const;
    AcGeVector3d&  set         (double, double, double);
    AcGeVector3d&  set         (const AcGePlanarEnt&, const AcGeVector2d&); 

    // Convert to the matrix of translation.
    //
    operator       AcGeMatrix3d() const;

    // Co-ordinates.
    //
    double         x, y, z;
};

// Creates the identity vector.
//
inline
AcGeVector3d::AcGeVector3d() : x(0.0), y(0.0), z(0.0)
{
}

inline
AcGeVector3d::AcGeVector3d(const AcGeVector3d& src) : x(src.x),y(src.y),z(src.z)
{
}

// Creates a vector intialized to ( xx, yy, zz ).
//
inline
AcGeVector3d::AcGeVector3d(double xx, double yy, double zz) : x(xx),y(yy),z(zz)
{
}

// Returns a vector that is formed from adding the components of
// this vector with `v'.
//
inline AcGeVector3d
AcGeVector3d::operator + (const AcGeVector3d& v) const
{
    return AcGeVector3d (x + v.x, y + v.y, z + v.z);
}

// This is equivalent to the statement `thisVec = thisVec + v;'
//
inline AcGeVector3d&
AcGeVector3d::operator += (const AcGeVector3d& v)
{
    x += v.x;
    y += v.y;
    z += v.z;
    return *this;
}

// Using this operator is equivalent to using `thisVec + (-v);'
//
inline AcGeVector3d
AcGeVector3d::operator - (const AcGeVector3d& v) const
{
    return AcGeVector3d (x - v.x, y - v.y, z - v.z);
}

// This is equivalent to the statement `thisVec = thisVec - v;'
//
inline AcGeVector3d&
AcGeVector3d::operator -= (const AcGeVector3d& v)
{
    x -= v.x;
    y -= v.y;
    z -= v.z;
    return *this;
}

inline AcGeVector3d&
AcGeVector3d::setToSum(const AcGeVector3d& v1, const AcGeVector3d& v2)
{
    x = v1.x + v2.x;
    y = v1.y + v2.y;
    z = v1.z + v2.z;
    return *this;
}

// Returns a vector that is formed by negating each of the components
// of this vector.
//
inline AcGeVector3d
AcGeVector3d::operator - () const
{
    return AcGeVector3d (-x, -y, -z);
}

// `v.negate()' is equivalent to the statement `v = -v;'
//
inline AcGeVector3d&
AcGeVector3d::negate()
{
    x = -x;
    y = -y;
    z = -z;
    return *this;
}

// This operator returns a vector that is the scalar product of
// `s' and this vector.
//
inline AcGeVector3d
AcGeVector3d::operator * (double s) const
{
    return AcGeVector3d (x * s, y * s, z * s);
}

// This operator returns a vector that is the scalar product of
// `s' and the vector `v'.
//
inline AcGeVector3d
operator * (double s, const AcGeVector3d& v)
{
    return AcGeVector3d (s * v.x, s * v.y, s * v.z);
}

// This is equivalent to the statement `v = v * s'.
//
inline AcGeVector3d&
AcGeVector3d::operator *= (double s)
{
    x *= s;
    y *= s;
    z *= s;
    return *this;
}

inline AcGeVector3d&
AcGeVector3d::setToProduct(const AcGeVector3d& v, double s)
{
    x = s * v.x;
    y = s * v.y;
    z = s * v.z;
    return *this;
}

// Returns a vector such that each of the coordinates of this vector
// have been divided by val.
//
inline AcGeVector3d
AcGeVector3d::operator / (double val) const
{
    return AcGeVector3d (x/val, y/val, z/val);
}

// This is equivalent to the statement `v = v / val;'
// Each coordinate of this vector is divided by val.
//
inline AcGeVector3d&
AcGeVector3d::operator /= (double val)
{
    x /= val;
    y /= val;
    z /= val;
    return *this;
}

// Returns the square of the Euclidean length of this vector.
//
inline double
AcGeVector3d::lengthSqrd() const
{
    return x*x + y*y + z*z;
}

// Returns the dot product of this vector and `v'.
//
inline double
AcGeVector3d::dotProduct(const AcGeVector3d& v) const
{
    return x * v.x + y * v.y + z * v.z;
}

// Sets the vector to ( xx, yy, zz ).
//
inline AcGeVector3d&
AcGeVector3d::set(double xx, double yy, double zz)
{
    x = xx;
    y = yy;
    z = zz;
    return *this;
}

// Indexes the vector as if it were an array.  `x' is index `0',
// `y' is index `1' and `z' is index `2'.
//
inline double
AcGeVector3d::operator [] (unsigned int i) const
{
    return *(&x+i);
}

inline double& AcGeVector3d::operator [] (unsigned int i)
{
    return *(&x+i);
}

#endif
